@@ -1,5 +1,6 @@
 ###########################################################################/**
 # @RdocFunction devOptions
+# @alias getDevOption
 #
 # @title "Gets the default device options"
 #
@@ -17,6 +18,8 @@
 #   \item{custom}{If @TRUE, also the default settings specific to this
 #      function is returned. For more details, see below.}
 #   \item{special}{A @logical.  For more details, see below.}
+#   \item{inherits}{If @TRUE, the global option is used if the
+#      type-specific is not set (or @NULL).}
 #   \item{drop}{If @TRUE and only one device type is queried, then
 #      a @list is returned, otherwise a @matrix.}
 #   \item{options}{Optional named @list of settings.}
@@ -58,35 +61,17 @@
 # @keyword device
 # @keyword utilities
 #*/###########################################################################
-devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "CairoX11", "eps", "jpeg", "jpeg2", "pdf", "pictex", "png", "png2", "postscript", "quartz", "svg", "tiff", "win.metafile", "windows", "x11", "X11", "xfig"), custom=TRUE, special=TRUE, drop=TRUE, options=list(), ..., reset=FALSE) {
+devOptions <- function(type=NULL, custom=TRUE, special=TRUE, inherits=FALSE, drop=TRUE, options=list(), ..., reset=FALSE) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Local setups
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   devList <- list(
-    bmp=c("grDevices::bmp"),
-    cairo_pdf=c("grDevices::cairo_pdf"),
-    cairo_ps=c("grDevices::cairo_ps"),
-##    CairoPNG=c("Cairo::CairoPNG"),  ## If used, gets extension CairoPNG.
-    CairoWin=c("Cairo::CairoWin"),
-    CairoX11=c("Cairo::CairoX11"),
-    eps=c("eps", "grDevices::postscript"),
-##    JavaGD=c("JavaGD::JavaGD"),
-    jpeg=c("grDevices::jpeg"),
-    jpeg2=c("jpeg2", "grDevices::bitmap", "grDevices::postscript"),
-    pdf=c("grDevices::pdf"),
-    pictex=c("grDevices::pictex"),
-    png=c("grDevices::png"),
-    png2=c("png2", "grDevices::bitmap", "grDevices::postscript"),
-    postscript=c("grDevices::postscript"),
-    quartz=c("grDevices::quartz"),
-    svg=c("grDevices::svg"),
-    tiff=c("grDevices::tiff"),
-    win.metafile=c("grDevices::win.metafile"),
-    windows=c("grDevices::windows"),
-    x11=c("grDevices::x11"),
-    X11=c("grDevices::X11"),
-    xfig=c("grDevices::xfig")
-  );
+    ## Global device options
+    "*"=NA_character_
+  )
+  ## All known and supported graphics devices on this system
+  devList <- c(devList, devAll())
+  knownTypes <- names(devList)
 
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -95,14 +80,23 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
   if (.Platform$OS.type == "windows") {
     # To please R CMD check
     windows.options <- NULL; rm(list="windows.options");
-    x11.options <- windows.options;
-    X11.options <- windows.options;
+    x11.options <- grDevices::windows.options;
+    X11.options <- x11.options;
+  }
+
+
+  # Global options; only used for its ability to reset them
+  "*.options" <- function(..., reset=FALSE) {
+    if (reset) {
+      devOptions(type="*", sep=",", path="figures", field=NULL, force=TRUE)
+    }
   }
 
 
   # A template for a dummy device options function.
-  getNnnOptions <- function(type, ...) {
+  getNnnOptions <- function(type, fallbacks=FALSE, ...) {
     optList <- list(
+      "*"="*.options",
       eps="ps.options",
       jpeg2="ps.options",
       pdf="pdf.options",
@@ -114,6 +108,14 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
       X11="x11.options"
     );
 
+    if (fallbacks) {
+      optList <- c(optList, list(
+        "CairoPDF"="pdf.options",
+        "CairoPS"="ps.options",
+        "CairoX11"="x11.options"
+      ))
+    }
+
     dummy <- function(...) { list(); }
 
     if (!is.element(type, names(optList))) {
@@ -282,6 +284,11 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Validate arguments
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Argument 'type':
+  if (is.null(type)) {
+    type <- names(devList)
+  }
+
   # Argument 'options':
   if (!is.list(options)) {
     throw("Argument 'options' must be a list: ", class(options)[1L]);
@@ -291,6 +298,12 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
     throw("Argument 'options' must be a named list.");
   }
 
+  # Argument 'inherits':
+  inherits <- as.logical(inherits);
+
+  # Argument 'reset':
+  reset <- as.logical(reset);
+
   # Additional arguments
   args <- list(...);
   nargs <- length(args);
@@ -298,8 +311,9 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
     throw("Optional ('...') arguments must be named.");
   }
 
+  # Append/overwrite 'options' with the named argument, e.g. field=NULL
   for (key in names(args)) {
-    options[[key]] <- args[[key]];
+    options[key] <- list(args[[key]]);
   }
   nopts <- length(options);
   # Not needed anymore
@@ -307,19 +321,25 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
 
   # Argument 'type':
   if (missing(type) || length(type) == 0L) {
-    knownTypes <- eval(formals(devOptions)$type);
     if (nopts > 0L) {
       throw("Cannot set device options. Argument 'type' is missing or NULL. Should be one of: ", paste(sprintf("'%s'", knownTypes), collapse=", "));
     }
 
-    res <- devOptions(type=knownTypes, custom=custom, special=special, drop=drop, reset=reset);
+    res <- devOptions(type=knownTypes, custom=custom, special=special, inherits=inherits, drop=drop, reset=reset);
     if (reset) {
       return(invisible(res));
     } else {
       return(res);
     }
+  } else if (is.character(type)) {
+    # Expand by regexp matching, iff any
+    type <- .devTypeName(type, pattern=TRUE, knownTypes=knownTypes);
   }
 
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # Vectorized call?
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (length(type) > 1L) {
     if (nopts > 0L) {
       throw("Cannot set device options for more than one devices at the time: ", hpaste(sprintf("'%s'", type), collapse=", "));
@@ -327,7 +347,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
 
     # Support vector of 'type':s
     types <- type;
-    res <- lapply(types, FUN=devOptions, drop=TRUE);
+    res <- lapply(types, FUN=devOptions, inherits=inherits, drop=TRUE);
     fields <- lapply(res, FUN=function(opts) names(opts));
     fields <- unique(unlist(fields, use.names=FALSE));
     opts <- lapply(res, FUN=function(x) x[fields]);
@@ -338,19 +358,27 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
     ##class(opts) <- c("DeviceOptions", class(opts));
     return(opts);
   }
+
+
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+  # A single device at this point
+  # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (is.function(type)) {
     # Try to find name of device function
     type <- findDeviceFunction(fcn=type);
   }
   if (is.character(type)) {
-    type <- .devTypeName(type);
-    type <- match.arg(type);
+    type <- match.arg(type, choices=knownTypes);
   }
 
   if (!is.element(type, names(devList))) {
     throw("Cannot query/modify device options. Unknown device: ", type);
   }
 
+  if (inherits) {
+    throw("Argument inherits=TRUE is not yet supported.");
+  }
+
 
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Locate the nnn.options() function for this type of device
@@ -394,6 +422,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
   # Get builtin device options, if available
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Only for certain devices...
+  nnn.options <- getNnnOptions(type, fallbacks=TRUE);
   opts <- nnn.options();
 
 
@@ -404,6 +433,7 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
   defArgs <- Reduce(append, defArgs);
   # Drop '...'
   defArgs <- defArgs[names(defArgs) != "..."];
+  # Drop missing
   # Drop overridden values
   defArgs <- defArgs[!duplicated(names(defArgs), fromLast=TRUE)];
 
@@ -445,13 +475,13 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
   # Adjust for special cases?
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   if (special) {
-    if (is.element(type, c("eps", "postscript"))) {
+    if (is.element(type, c("eps", "postscript", "CairoPS"))) {
       sizes <- c("a4", "executive", "legal", "letter");
       dim <- getSpecialDimensions(opts, sizes);
     } else if (type == "xfig") {
       sizes <- c("a4", "legal", "letter");
       dim <- getSpecialDimensions(opts, sizes);
-    } else if (type == "pdf") {
+    } else if (is.element(type, c("pdf", "CairoPDF"))) {
       sizes <- c("a4", "a4r", "executive", "legal", "letter", "USr");
       dim <- getSpecialDimensions(opts, sizes);
     } else {
@@ -481,8 +511,64 @@ devOptions <- function(type=c("bmp", "cairo_pdf", "cairo_ps", "CairoWin", "Cairo
 } # devOptions()
 
 
+getDevOption <- function(type, name, default=NULL, inherits=TRUE, ..., old=TRUE) {
+  # BACKWARD COMPATIBILITY:
+  if (isTRUE(old)) old <- sprintf("devEval/args/%s", name)
+  if (is.character(old)) {
+    .importOldGlobalOption(name, from=old, remove=TRUE)
+  }
+
+  # Get options
+  if (is.character(type) && type != "*") {
+    value <- devOptions(type=type, ...)[[name]]
+  } else {
+    value <- NULL
+  }
+
+  # Fallback to global options?
+  if (is.null(value) && inherits) {
+    value <- devOptions(type="*", ...)[[name]]
+  }
+
+  # Use default value?
+  if (is.null(value)) value <- default
+
+  value
+} # getDevOption()
+
+
+# BACKWARD COMPATIBILITY for old-style R options().
+.importOldGlobalOption <- function(name, from, remove=TRUE, ...) {
+  # Nothing to do?
+  if (!is.element(from, names(options()))) return()
+
+  # (a) Get old-style option value
+  opt <- options(from);
+
+  # (b) Assign to global device options instead
+  names(opt) <- name;
+  do.call(devOptions, args=list(type="*", options=opt));
+
+  # (c) Delete old-style option and never look back
+  if (remove) setOption(from, NULL);
+} # .importOldGlobalOption()
+
+
 ############################################################################
 # HISTORY:
+# 2014-09-16
+# o Added support for regular expression matching for argument 'type'.
+# o BUG FIX: devOptions() returned the incorrect options for device
+#   types "eps", "jpg2" and "png2" if package was not attached.
+# 2014-09-15
+# o Added "favicon" as a known type.
+# 2014-09-12
+# o Added support for global options via devOptions("*").
+# o BUG FIX: On Windows, devOptions() assumed that the 'grDevices'
+#   package was attached.
+# o BUG FIX: devOptions(type, name=NULL) did not assign the option NULL.
+# 2014-09-12
+# o Added getDevOption().
 # 2013-12-08
 # o BUG FIX: devOptions(types) would drop all options for combinations
 #   devices types that have identical sets of options, e.g.
