@@ -435,7 +435,9 @@ devDone <- function(which=dev.cur(), ...) {
     type <- tolower(names(which));
     type <- gsub(":.*", "", type);
 
-    isOnScreen <- (is.element(type, deviceIsInteractive()));
+    knownInteractive <- deviceIsInteractive();
+    knownInteractive <- tolower(knownInteractive);
+    isOnScreen <- (is.element(type, knownInteractive));
     if (!isOnScreen)
       devOff(which);
   }
@@ -475,6 +477,14 @@ devDone <- function(which=dev.cur(), ...) {
 # @keyword utilities
 #*/###########################################################################
 devIsInteractive <- function(types, ...) {
+  # Known interactive devices
+  knownInteractive <- grDevices::deviceIsInteractive();
+  knownInteractive <- c(knownInteractive, "CairoWin", "CairoX11");
+##  knownInteractive <- c(knownInteractive, "Cairo");
+
+  # Return all known?
+  if (missing(types)) return(knownInteractive);
+
   # Nothing to do?
   if (length(types) == 0L) {
     res <- logical(0L);
@@ -495,9 +505,6 @@ devIsInteractive <- function(types, ...) {
   # Investigate one type below
   type0 <- type <- types;
 
-  # A known interactive device?
-  knownInteractive <- grDevices::deviceIsInteractive();
-
   if (is.function(type)) {
     for (name in knownInteractive) {
       if (exists(name, mode="function")) {
@@ -516,6 +523,7 @@ devIsInteractive <- function(types, ...) {
     # Device type aliases?
     type <- .devTypeName(type);
     type <- tolower(type);
+
     # An known one?
     res <- is.element(type, tolower(knownInteractive));
     names(res) <- type0;
@@ -526,6 +534,174 @@ devIsInteractive <- function(types, ...) {
 
 
 
+devAll <- local({
+  # Memoize results, because this function is called many many
+  # times either directly or indirectly by various functions.
+  .devAll <- NULL
+
+  isFALSE <- function(x) identical(FALSE, unname(x))
+
+  base_capabilities <- local({
+    res <- base::capabilities()
+    function(force=FALSE) {
+      if (force) res <<- base::capabilities()
+      res
+    }
+  })
+
+  supports <- function(type, pkg="grDevices") {
+    capabilities <- function() character(0L)
+    if (isPackageInstalled(pkg)) {
+      if (pkg == "Cairo") {
+        ns <- getNamespace("Cairo")
+        Cairo.capabilities <- get("Cairo.capabilities", envir=ns, mode="function")
+        capabilities <- function() {
+          res <- Cairo.capabilities()
+          names <- sprintf("Cairo%s", toupper(names(res)))
+          names <- gsub("WIN", "Win", names, fixed=TRUE)
+          names(res) <- names
+          res
+        }
+      } else if (pkg == "grDevices") {
+        capabilities <- base_capabilities
+      }
+    }
+    x <- capabilities()
+    !isFALSE(x[type])
+  } # supports()
+
+
+  function(force=FALSE, ...) {
+    res <- .devAll
+    if (force || is.null(res)) {
+      if (force) base_capabilities(force=TRUE)
+
+      # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+      # Setup all possible devices
+      # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+      res <- list()
+
+      ## grDevices
+      res <- c(res, list(
+        bmp          = "grDevices::bmp",
+        jpeg         = "grDevices::jpeg",
+        pdf          = "grDevices::pdf",
+        pictex       = "grDevices::pictex",
+        png          = "grDevices::png",
+        postscript   = "grDevices::postscript",
+        quartz       = "grDevices::quartz",
+        svg          = c("grDevices::svg", "needs::cairo"),
+        cairo_pdf    = c("grDevices::cairo_pdf", "needs::cairo"),
+        cairo_ps     = c("grDevices::cairo_ps", "needs::cairo"),
+        tiff         = "grDevices::tiff",
+        win.metafile = "grDevices::win.metafile",
+        xfig         = "grDevices::xfig"
+      ))
+      res <- c(res, list(
+        windows    = "grDevices::windows",
+        x11        = "grDevices::x11",
+        X11        = "grDevices::X11"
+      ))
+
+      # R.devices
+      res <- c(res, list(
+        eps     = c("R.devices::eps",
+                    "grDevices::postscript"),
+        favicon = c("R.devices::favicon",
+                    "grDevices::png"),
+        jpeg2   = c("R.devices::jpeg2",
+                    "grDevices::bitmap", "grDevices::postscript"),
+        png2    = c("R.devices::png2",
+                    "grDevices::bitmap", "grDevices::postscript")
+      ))
+
+      ## Cairo package
+      res <- c(res, list(
+        CairoPDF  = c("Cairo::CairoPDF",
+                      "grDevices::pdf"),
+        CairoPS   = c("Cairo::CairoPS",
+                      "grDevices::postscript"),
+        CairoPNG  = c("Cairo::CairoPNG",
+                      "grDevices::png"),
+        CairoJPEG = c("Cairo::CairoJPEG",
+                      "grDevices::jpeg"),
+        CairoTIFF = c("Cairo::CairoTIFF",
+                      "grDevices::tiff"),
+        CairoSVG  = c("Cairo::CairoSVG",
+                      "grDevices::svg")
+      ))
+      res <- c(res, list(
+        CairoWin  = c("Cairo::CairoWin",
+                      "grDevices::windows"),
+        CairoX11  = c("Cairo::CairoX11",
+                      "grDevices::x11")
+      ))
+
+      ## JavaGD
+      ## JavaGD=c("JavaGD::JavaGD")
+
+
+      # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+      # Drop devices this system is not capable of
+      # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+      for (type in names(res)) {
+        # Assume it is supported, unless...
+        supported <- TRUE
+
+        for (fcn in res[[type]]) {
+          pattern <- "^(.+)::(.+)$"
+          pkg <- gsub(pattern, "\\1", fcn)
+          name <- gsub(pattern, "\\2", fcn)
+
+          if (pkg == "needs") {
+            if (!supports(name)) {
+              supported <- FALSE
+              break
+            }
+            next
+          }
+
+          if (!isPackageInstalled(pkg)) {
+            supported <- FALSE
+            break
+          }
+
+          ns <- getNamespace(pkg)
+          if (!exists(name, envir=ns, mode="function")) {
+            supported <- FALSE
+            break
+          }
+
+          if (!supports(type, pkg=pkg)) {
+            supported <- FALSE
+            break
+          }
+        } # for (fcn ...)
+
+        # Drop any 'needs::*'
+        res[[type]] <- grep("^needs::", res[[type]], value=TRUE, invert=TRUE)
+
+        # Not supported?
+        if (!supported) res[[type]] <- NULL
+      } # for (type ...)
+
+
+      # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+      # Order by name
+      # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+      o <- order(names(res))
+      res <- res[o]
+
+      # Memoize
+      .devAll <<- res
+    } # if (force ...)
+
+    res
+  }
+}) # devAll()
+
+
+
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # BEGIN: Local functions
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -590,24 +766,70 @@ devIsInteractive <- function(types, ...) {
 } # .devNextAvailable()
 
 
-.devTypeName <- function(types, ...) {
+.devTypeName <- function(types, pattern=FALSE, knownTypes=names(devAll()), ...) {
   # Nothing todo?
   if (!is.character(types)) {
     return(types);
   }
 
-  types0 <- types;
-#  types <- tolower(types);
+  names <- names(types);
+  if (is.null(names)) {
+    names(types) <- types;
+  }
+
+  # Match to known set of device types by regular expression?
+  if (pattern) {
+    types <- as.list(types);
+    for (kk in seq_along(types)) {
+      typeKK <- types[[kk]];
+      pattern <- sprintf("^%s$", typeKK);
+      idxs <- grep(pattern, knownTypes);
+      if (length(idxs) > 0L) {
+        typesKK <- knownTypes[idxs];
+        names(typesKK) <- rep(typeKK, length=length(typesKK));
+        types[[kk]] <- typesKK;
+      } else {
+        names(types[[kk]]) <- typeKK;
+      }
+    } # for (kk ...)
+    types <- unlist(types, use.names=TRUE);
+  }
 
   # Common aliases
+  names <- names(types);
   types[types == "jpg"] <- "jpeg";
   types[types == "ps"] <- "postscript";
-
-  names(types) <- types0;
+  names(types) <- names;
 
   types;
 } # .devTypeName()
 
+.devTypeExt <- function(types, ...) {
+  types <- as.character(types);
+  exts <- types;
+
+  ## Cairo package
+  pattern <- "^Cairo(JPEG|PDF|PNG|PS|SVG|TIFF)$";
+  idxs <- grep(pattern, exts);
+  exts[idxs] <- tolower(gsub(pattern, "\\1", exts[idxs]));
+
+  ## cairo_* devices
+  pattern <- "^cairo_(pdf|ps)$";
+  exts <- gsub(pattern, "\\1", exts);
+
+  ## Recognize types of any case. Always return in lower case.
+  exts <- tolower(exts);
+
+  # Common type-to-extension conversions
+  exts[exts == "win.metafile"] <- "wmf";
+  exts[exts == "png2"] <- "png";
+  exts[exts == "jpeg"] <- "jpg";
+  exts[exts == "jpeg2"] <- "jpg";
+  exts[exts == "postscript"] <- "ps";
+
+  exts
+} # .devTypeExt()
+
 
 .devEqualTypes <- (function() {
    # Recorded known results
@@ -662,8 +884,22 @@ devIsInteractive <- function(types, ...) {
 # END: Local functions
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+
 ############################################################################
 # HISTORY:
+# 2014-10-17
+# o SPEEDUP: Made devAll() memoize base::capabilities() results, which
+#   gives a significant speedup when for instance an X11 server times out.
+# 2014-09-30
+# o BUG FIX: devDone() would close some devices despite them being
+#   on screen/interactive devices, e.g. an x11 device.
+# 2014-09-17
+# o Added devAll().
+# 2014-09-16
+# o Now devIsInteractive() without arguments returns all known
+#   interactive devices.
+# 2014-09-11
+# o Added .devTypeExt().
 # 2014-04-27
 # o Added .devEqualTypes().
 # 2013-10-29
