@@ -28,17 +28,16 @@
 # }
 #
 # \usage{
-#   toBMP(name, ..., field=getOption("devEval/args/field", "pathname"))
-#   toEMF(name, ..., ext="emf", field=getOption("devEval/args/field", "pathname"))
-#   toPDF(name, ..., field=getOption("devEval/args/field", "pathname"))
-#   toPNG(name, ..., field=getOption("devEval/args/field", "pathname"))
-#   toSVG(name, ..., field=getOption("devEval/args/field", "pathname"))
-#   toTIFF(name, ..., field=getOption("devEval/args/field", "pathname"))
-#   toWMF(name, ..., ext="wmf", field=getOption("devEval/args/field", "pathname"))
-#   %% toCairoPNG(name, ..., field=getOption("devEval/args/field", "pathname"))
+#   toBMP(name, ...)
+#   toPDF(name, ...)
+#   toPNG(name, ...)
+#   toSVG(name, ...)
+#   toTIFF(name, ...)
+#   toEMF(name, ..., ext="emf")
+#   toWMF(name, ..., ext="wmf")
 #
-#   toFavicon(..., name="favicon", type="png", width=32L, height=width,
-#                  par=list(mar=c(0,0,0,0), bg="transparent"))
+#   toFavicon(..., name="favicon", ext="png",
+#             field=getDevOption("favicon", "field", default="htmlscript"))
 #
 #   % Interactive/screen devices
 #   toDefault(name, ...)
@@ -56,15 +55,17 @@
 #   \item{name}{A @character string specifying the name of the image file.}
 #   \item{...}{Additional arguments passed to @see "devEval", e.g.
 #      \code{tags} and \code{aspectRatio}.}
-#   \item{ext}{A @character string specifying the filename extension.}
-#   \item{field}{Passed to @see "devEval".}
-#   \item{width, height, type, par}{Passed to @see "devEval".}
+#   \item{ext, field}{Passed to @see "devEval".}
 # }
 #
 # \value{
-#   Returns by default the so called \emph{fullname} of the generated
-#   image file.  The fullname of an file is the filename without the
-#   extension.
+#   Returns by default the @see "DevEvalProduct".
+#   For \code{toFavicon()} the default return value is a @character string.
+# }
+#
+# \details{
+#   Both \code{toEMF()} and \code{toWMF()} use the exact same graphics
+#   device and settings.  They only differ by filename exension.
 # }
 #
 # @author
@@ -78,50 +79,56 @@
 # @keyword device
 # @keyword utilities
 #*/###########################################################################
-toBMP <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="bmp", name=name, ..., field=field);
+toBMP <- function(name, ...) {
+  devEval(type="bmp", name=name, ...);
 }
 
-toEMF <- function(name, ..., ext="emf", field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="win.metafile", name=name, ..., ext=ext, field=field);
+toEMF <- function(name, ..., ext="emf") {
+  devEval(type="win.metafile", name=name, ..., ext=ext);
 }
 
-toWMF <- function(name, ..., ext="wmf", field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="win.metafile", name=name, ..., ext=ext, field=field);
+toWMF <- function(name, ..., ext="wmf") {
+  devEval(type="win.metafile", name=name, ..., ext=ext);
 }
 
-toEPS <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="eps", name=name, ..., field=field);
+toEPS <- function(name, ...) {
+  devEval(type="eps", name=name, ...);
 }
 
-toPDF <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="pdf", name=name, ..., field=field);
+toPDF <- function(name, ...) {
+  devEval(type="pdf", name=name, ...);
 }
 
-toPNG <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="png", name=name, ..., field=field);
+toPNG <- function(name, ...) {
+  devEval(type="png", name=name, ...);
 }
 
-toSVG <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="svg", name=name, ..., field=field);
+toSVG <- function(name, ...) {
+  devEval(type="svg", name=name, ...);
 }
 
-toTIFF <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-  devEval(type="tiff", name=name, ..., field=field);
+toTIFF <- function(name, ...) {
+  devEval(type="tiff", name=name, ...);
 }
 
-## toCairoPNG <- function(name, ..., field=getOption("devEval/args/field", "pathname")) {
-##   devEval(type="CairoPNG", name=name, ..., field=field);
-## }
-
-
-toFavicon <- function(..., name="favicon", type="png", width=32L, height=width, par=list(mar=c(0,0,0,0), bg="transparent")) {
-  # png() will generate a warning that "width=.. , height=.., are unlikely
-  # values in pixels" if width < 20.
-  suppressWarnings({
-    devEval(type=type, name=name, width=width, height=height, par=par, ...);
-  })
-}
+toFavicon <- function(..., name="favicon", ext="png", field=getDevOption("favicon", "field", default="htmlscript")) {
+  # Output as "<script>...</script>"?
+  if (identical(field, "htmlscript")) {
+    uri <- devEval(type="favicon", name=name, ext=ext, field="dataURI", ...);
+    script <- c(
+      "<script>",
+      " var link = document.createElement('link');",
+      " link.rel = 'icon';",
+      sprintf(' link.href = "%s"', uri),
+      " document.getElementsByTagName('head')[0].appendChild(link);",
+      "</script>",
+      ""
+    )
+    paste(script, collapse="\n")
+  } else {
+    devEval(type="favicon", name=name, ext=ext, field=field, ...);
+  }
+} # toFavicon()
 
 asDataURI <- function(..., mime=NULL) {
   img <- DevEvalFileProduct(...);
@@ -166,6 +173,10 @@ toCairoX11 <- function(name, ...) {
 
 ##############################################################################
 # HISTORY:
+# 2014-09-15
+# o Added support for toFavicon(..., field="htmlscript").  The device
+#   options are such that the default is to use "htmlscript".
+# o CLEANUP: toNnn():s no longer sets 'field' explitly.
 # 2014-08-29
 # o Changed default argument 'field' for toNNN() from "fullname" to
 #   "pathname" as in getOption("devEval/args/field", "pathname").
