@@ -1,4 +1,5 @@
-options("devEval/args/field"="*")
+# Return the DevEvalProduct object by default
+R.devices::devOptions("*", field="*")
 
 res <- R.devices::toPNG("foo", tags=c("a", "b"), aspectRatio=0.7, {
   plot(1:10)
